@@ -5,6 +5,8 @@
 """
 
 
+from __future__ import unicode_literals
+
 import logging
 
 from difflib import unified_diff
@@ -21,13 +23,12 @@ from mongoengine.errors import ValidationError, DoesNotExist
 
 from tabination.views import TabView
 
-from pure_pagination import Paginator, PageNotAnInteger
-from pure_pagination.mixins import PaginationMixin
-
 from upaas_admin.common.mixin import (
-    LoginRequiredMixin, AppTemplatesDirMixin, DetailTabView, MongoDetailView)
+    LoginRequiredMixin, AppTemplatesDirMixin, DetailTabView, MongoDetailView,
+    ModelRelatedPaginationMixin)
 from upaas_admin.apps.applications.mixin import (
-    OwnedAppsMixin, OwnedPackagesMixin, OwnedAppTasksMixin, AppActionView)
+    OwnedAppsMixin, OwnedPackagesMixin, OwnedAppTasksMixin,
+    OwnedAppDomainsMixin, AppActionView)
 from upaas_admin.apps.applications.models import (Application, Package,
                                                   ApplicationDomain)
 from upaas_admin.apps.applications.forms import (
@@ -46,27 +47,19 @@ log = logging.getLogger(__name__)
 
 
 class IndexView(LoginRequiredMixin, OwnedAppsMixin, AppTemplatesDirMixin,
-                PaginationMixin, TabView):
+                ModelRelatedPaginationMixin, TabView):
 
     template_name = 'index.html'
-    paginate_by = 10
     _is_tab = True
     tab_id = 'site_index'
     tab_group = 'users_index'
     tab_label = _('Applications')
 
-    def get(self, request, *args, **kwargs):
-        try:
-            page = request.GET.get('page', 1)
-        except PageNotAnInteger:
-            page = 1
-        self.object_list = request.user.applications
-        paginator = Paginator(self.object_list, self.paginate_by,
-                              request=request)
-        tasks = paginator.page(page)
-        context = self.get_context_data(object_list=tasks.object_list,
-                                        page_obj=tasks)
-        return self.render_to_response(context)
+    def get_object(self):
+        return None
+
+    def paginated_objects(self):
+        return self.request.user.applications
 
 
 class RegisterApplicationView(LoginRequiredMixin, AppTemplatesDirMixin,
@@ -114,6 +107,24 @@ class ApplicationDetailView(LoginRequiredMixin, OwnedAppsMixin,
     tab_label = _('Details')
 
 
+class ApplicationDomainsView(LoginRequiredMixin, OwnedAppsMixin,
+                             AppTemplatesDirMixin, ModelRelatedPaginationMixin,
+                             TabView, MongoDetailView):
+
+    template_name = 'domains.html'
+    model = Application
+    slug_field = 'id'
+    context_object_name = 'app'
+    context_paginated_objects_name = 'domains'
+    _is_tab = True
+    tab_id = 'app_domains'
+    tab_group = 'app_navigation'
+    tab_label = _('Domains')
+
+    def paginated_objects(self):
+        return self.object.custom_domains
+
+
 class ApplicationMetadataView(LoginRequiredMixin, OwnedAppsMixin,
                               AppTemplatesDirMixin, MongoDetailView,
                               DetailTabView):
@@ -137,33 +148,22 @@ class ApplicationMetadataView(LoginRequiredMixin, OwnedAppsMixin,
 
 
 class ApplicationPackagesView(LoginRequiredMixin, OwnedAppsMixin,
-                              AppTemplatesDirMixin, PaginationMixin,
-                              MongoDetailView, TabView):
+                              AppTemplatesDirMixin,
+                              ModelRelatedPaginationMixin, MongoDetailView,
+                              TabView):
 
     template_name = 'packages.html'
     model = Application
     slug_field = 'id'
     context_object_name = 'app'
-    paginate_by = 10
+    context_paginated_objects_name = 'packages'
     _is_tab = True
     tab_id = 'app_packages'
     tab_group = 'app_navigation'
     tab_label = _('Packages')
 
-    def get(self, request, *args, **kwargs):
-        try:
-            page = request.GET.get('page', 1)
-        except PageNotAnInteger:
-            page = 1
-        self.object = self.get_object()
-        self.object_list = Package.objects(application=self.object)
-        paginator = Paginator(self.object_list, self.paginate_by,
-                              request=request)
-        packages = paginator.page(page)
-        context = self.get_context_data(object=self.object,
-                                        packages=packages.object_list,
-                                        page_obj=packages)
-        return self.render_to_response(context)
+    def paginated_objects(self):
+        return Package.objects(application=self.object)
 
 
 class ApplicationInstancesView(LoginRequiredMixin, OwnedAppsMixin,
@@ -198,34 +198,25 @@ class ApplicationStatsView(LoginRequiredMixin, OwnedAppsMixin,
 
 
 class ApplicationTasksView(LoginRequiredMixin, OwnedAppsMixin,
-                           AppTemplatesDirMixin, PaginationMixin,
+                           AppTemplatesDirMixin, ModelRelatedPaginationMixin,
                            MongoDetailView, DetailTabView):
 
     template_name = 'tasks.html'
     model = Application
     slug_field = 'id'
     context_object_name = 'app'
-    paginate_by = 10
+    context_paginated_objects_name = 'tasks'
     tab_id = 'app_tasks'
     tab_group = 'app_navigation'
     tab_label = _('Tasks')
 
-    def get(self, request, *args, **kwargs):
-        try:
-            page = request.GET.get('page', 1)
-        except PageNotAnInteger:
-            page = 1
-        self.object = self.get_object()
-        self.object_list = self.object.tasks.order_by('-date_created',
-                                                      'parent')
-        paginator = Paginator(self.object_list, self.paginate_by,
-                              request=request)
-        tasks = paginator.page(page)
-        context = self.get_context_data(object=self.object,
-                                        tasks=tasks.object_list,
-                                        task_statuses=TaskStatus,
-                                        page_obj=tasks)
-        return self.render_to_response(context)
+    def paginated_objects(self):
+        return self.object.tasks.order_by('-date_created', 'parent')
+
+    def get_context_data(self, **kwargs):
+        context = super(ApplicationTasksView, self).get_context_data(**kwargs)
+        context['task_statuses'] = TaskStatus
+        return context
 
 
 class ApplicationTaskDetailsView(LoginRequiredMixin, OwnedAppTasksMixin,
@@ -266,20 +257,20 @@ class StartApplicationView(LoginRequiredMixin, OwnedAppsMixin,
         self.app = self.get_object()
         if self.app.run_plan:
             return application_error(request, self.app,
-                                     _(u"Application is already started"))
+                                     _("Application is already started"))
         elif not self.app.can_start:
             return application_error(request, self.app,
-                                     _(u"Application cannot be started yet"))
+                                     _("Application cannot be started yet"))
         return super(StartApplicationView, self).get(request, *args, **kwargs)
 
     def post(self, request, *args, **kwargs):
         self.app = self.get_object()
         if self.app.run_plan:
             return application_error(request, self.app,
-                                     _(u"Application is already started"))
+                                     _("Application is already started"))
         elif not self.app.can_start:
             return application_error(request, self.app,
-                                     _(u"Application cannot be started yet"))
+                                     _("Application cannot be started yet"))
         return super(StartApplicationView, self).post(request, *args, **kwargs)
 
     def get_object(self, queryset=None):
@@ -325,7 +316,7 @@ class EditApplicationRunPlanView(LoginRequiredMixin, OwnedAppsMixin,
         self.app = self.get_object()
         if not self.app.run_plan:
             return application_error(request, self.app,
-                                     _(u"Application is stopped"))
+                                     _("Application is stopped"))
         return super(EditApplicationRunPlanView, self).get(request, *args,
                                                            **kwargs)
 
@@ -333,7 +324,7 @@ class EditApplicationRunPlanView(LoginRequiredMixin, OwnedAppsMixin,
         self.app = self.get_object()
         if not self.app.run_plan:
             return application_error(request, self.app,
-                                     _(u"Application is stopped"))
+                                     _("Application is stopped"))
         return super(EditApplicationRunPlanView, self).post(request, *args,
                                                             **kwargs)
 
@@ -387,7 +378,7 @@ class StopApplicationView(AppActionView):
     def validate_action(self, request):
         if not self.object.run_plan:
             return application_error(request, self.object,
-                                     _(u"Application is already stopped"))
+                                     _("Application is already stopped"))
 
     def action(self, form):
         if self.object:
@@ -408,8 +399,8 @@ class PackageDetailView(LoginRequiredMixin, OwnedPackagesMixin,
             context['app'] = self.object.application
         context['metadiff'] = list(unified_diff(
             self.object.application.metadata.splitlines(1),
-            self.object.metadata.splitlines(1), fromfile=__(u"Application"),
-            tofile=__(u"Package")))
+            self.object.metadata.splitlines(1), fromfile=__("Application"),
+            tofile=__("Package")))
         return context
 
 
@@ -451,7 +442,7 @@ class PackageDeleteView(LoginRequiredMixin, OwnedPackagesMixin,
             return HttpResponseRedirect(self.get_success_url())
         else:
             return application_error(self.request, self.object.application,
-                                     _(u"Package in use"))
+                                     _("Package in use"))
 
 
 class BuildPackageView(AppActionView):
@@ -464,7 +455,9 @@ class BuildPackageView(AppActionView):
     task = None
 
     def get_success_url(self):
-        return reverse('app_task_details', args=[self.task.safe_id])
+        if self.task:
+            return reverse('app_task_details', args=[self.task.safe_id])
+        return reverse('app_details', args=[self.object.safe_id])
 
     def action(self, form):
         if self.object and self.object.metadata:
@@ -486,8 +479,8 @@ class RollbackApplicationView(OwnedPackagesMixin, AppActionView):
     def validate_action(self, request):
         if self.object == self.object.application.current_package:
             return application_error(
-                request, self.object, _(u"Selected package is already current "
-                                        u"for this application"))
+                request, self.object, _("Selected package is already current "
+                                        "for this application"))
 
     def action(self, form):
         if self.object:
@@ -546,17 +539,10 @@ class DownloadPackageMetadataView(LoginRequiredMixin, OwnedPackagesMixin,
         return response
 
 
-class ApplicationDomainsView(LoginRequiredMixin, OwnedAppsMixin,
-                             AppTemplatesDirMixin, PaginationMixin,
-                             MongoDetailView):
+class AssignApplicationDomainView(LoginRequiredMixin, OwnedAppsMixin,
+                                  AppTemplatesDirMixin, CreateView,
+                                  SingleObjectMixin):
 
-    template_name = 'domains.html'
-    model = Application
-    slug_field = 'id'
-    context_object_name = 'app'
-
-
-class AssignApplicationDomainView(AppActionView):
     template_name = 'assign_domain.html'
     model = Application
     slug_field = 'id'
@@ -564,42 +550,76 @@ class AssignApplicationDomainView(AppActionView):
     form_class = AssignApplicatiomDomainForm
 
     def get_success_url(self):
-        return reverse('app_domains', args=[self.object.safe_id])
+        return reverse('app_domains', args=[self.app.safe_id])
+
+    def get(self, request, *args, **kwargs):
+        self.app = self.get_object()
+        return super(AssignApplicationDomainView, self).get(request, *args,
+                                                            **kwargs)
+
+    def post(self, request, *args, **kwargs):
+        self.app = self.get_object()
+        return super(AssignApplicationDomainView, self).post(request, *args,
+                                                             **kwargs)
+
+    def get_object(self, queryset=None):
+        try:
+            return super(AssignApplicationDomainView, self).get_object(
+                queryset=queryset)
+        except (ValidationError, DoesNotExist):
+            raise Http404
+
+    def get_context_data(self, **kwargs):
+        context = super(AssignApplicationDomainView, self).get_context_data(
+            **kwargs)
+        context['app'] = self.app
+        return context
 
     def get_form(self, form_class):
         form = super(AssignApplicationDomainView, self).get_form(form_class)
-        form.app = self.object
+        form.app = self.app
         form.domain_validated = False
         form.needs_validation = settings.UPAAS_CONFIG.apps.domains.validation
         return form
 
     def form_valid(self, form):
-        domain = ApplicationDomain(name=form.cleaned_data['domain'],
-                                   validated=form.domain_validated)
-        self.object.update(add_to_set__domains=domain)
-        self.object.update_application()
-        return super(AssignApplicationDomainView, self).form_valid(form)
+        form.instance.application = self.app
+        form.instance.validated = form.domain_validated
+        ret = super(AssignApplicationDomainView, self).form_valid(form)
+        self.app.update_application()
+        return ret
 
 
-class RemoveApplicationDomainView(AppActionView):
+class RemoveApplicationDomainView(LoginRequiredMixin, OwnedAppDomainsMixin,
+                                  AppTemplatesDirMixin, FormView, DeleteView,
+                                  MongoDetailView):
+
     template_name = 'remove_domain.html'
-    model = Application
+    model = ApplicationDomain
     slug_field = 'id'
-    context_object_name = 'app'
+    context_object_name = 'domain'
     form_class = RemoveApplicatiomDomainForm
-
-    def validate_action(self, request):
-        if self.kwargs.get('domain') not in [
-                d.name for d in self.object.domains]:
-            raise Http404
+    application = None
 
     def get_success_url(self):
-        return reverse('app_domains', args=[self.object.safe_id])
+        return reverse('app_domains', args=[self.application.safe_id])
 
-    def get_initial(self):
-        return {'domain': self.kwargs.get('domain')}
+    def get_context_data(self, **kwargs):
+        context = super(RemoveApplicationDomainView, self).get_context_data(
+            **kwargs)
+        if self.object:
+            self.application = self.object.application
+            context['app'] = self.application
+        context['form'] = self.get_form(self.get_form_class())
+        return context
+
+    def post(self, *args, **kwargs):
+        self.object = self.get_object()
+        return FormView.post(self, *args, **kwargs)
 
     def form_valid(self, form):
-        self.object.update(pull__domains__name=form.cleaned_data['domain'])
-        self.object.update_application()
-        return super(RemoveApplicationDomainView, self).form_valid(form)
+        self.get_context_data()
+        ret = self.delete(self.request)
+        if self.application:
+            self.application.update_application()
+        return ret
